/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_INNOLIDARSERVICE_INNOLIDARSERVICE_PROXY_H
#define JAC_AP_INNOLIDARSERVICE_INNOLIDARSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/innolidarservice/innolidarservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace innolidarservice {
namespace proxy {
namespace events {
    using InnoLiDAR2LiDARPerceptionEvent = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LiDARPointCloudEvent = ara::com::internal::proxy::event::EventAdapter<::LidarPointCloudNew>;
    static constexpr ara::com::internal::EntityId InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId = 57993U; //InnoLiDAR2LiDARPerceptionEvent_event_hash
    static constexpr ara::com::internal::EntityId InnoLiDARServiceLiDARPointCloudEventId = 20619U; //LiDARPointCloudEvent_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class InnoLiDARServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              InnoLiDAR2LiDARPerceptionEvent(proxyAdapter->GetProxy(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LiDARPointCloudEvent(proxyAdapter->GetProxy(), events::InnoLiDARServiceLiDARPointCloudEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::InnoLiDAR2LiDARPerceptionEvent GetInnoLiDAR2LiDARPerceptionEvent() noexcept { return std::move(InnoLiDAR2LiDARPerceptionEvent); }
        events::LiDARPointCloudEvent GetLiDARPointCloudEvent() noexcept { return std::move(LiDARPointCloudEvent); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::InnoLiDAR2LiDARPerceptionEvent InnoLiDAR2LiDARPerceptionEvent;
        events::LiDARPointCloudEvent LiDARPointCloudEvent;
    };

    virtual ~InnoLiDARServiceProxy()
    {
        InnoLiDAR2LiDARPerceptionEvent.UnsetReceiveHandler();
        InnoLiDAR2LiDARPerceptionEvent.Unsubscribe();
        LiDARPointCloudEvent.UnsetReceiveHandler();
        LiDARPointCloudEvent.Unsubscribe();

    }

    explicit InnoLiDARServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, handle)),
          InnoLiDAR2LiDARPerceptionEvent(proxyAdapter->GetProxy(), events::InnoLiDARServiceInnoLiDAR2LiDARPerceptionEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LiDARPointCloudEvent(proxyAdapter->GetProxy(), events::InnoLiDARServiceLiDARPointCloudEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    InnoLiDARServiceProxy(const InnoLiDARServiceProxy&) = delete;
    InnoLiDARServiceProxy& operator=(const InnoLiDARServiceProxy&) = delete;

    InnoLiDARServiceProxy(InnoLiDARServiceProxy&&) = default;
    InnoLiDARServiceProxy& operator=(InnoLiDARServiceProxy&&) = default;
    InnoLiDARServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          InnoLiDAR2LiDARPerceptionEvent(token.GetInnoLiDAR2LiDARPerceptionEvent()),
          LiDARPointCloudEvent(token.GetLiDARPointCloudEvent()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<InnoLiDARServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::innolidarservice::InnoLiDARService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::InnoLiDAR2LiDARPerceptionEvent InnoLiDAR2LiDARPerceptionEvent;
    events::LiDARPointCloudEvent LiDARPointCloudEvent;
};
} // namespace proxy
} // namespace innolidarservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_INNOLIDARSERVICE_INNOLIDARSERVICE_PROXY_H
